#!/usr/bin/perl
#
# Update CGI for Endian Firewall
#
#
#        +-----------------------------------------------------------------------------+
#        | Endian Firewall                                                             |
#        +-----------------------------------------------------------------------------+
#        | Copyright (c) 2005-2006 Endian                                              |
#        |         Endian GmbH/Srl                                                     |
#        |         Bergweg 41 Via Monte                                                |
#        |         39057 Eppan/Appiano                                                 |
#        |         ITALIEN/ITALIA                                                      |
#        |         info@endian.it                                                      |
#        |                                                                             |
#        | This program is free software; you can redistribute it and/or               |
#        | modify it under the terms of the GNU General Public License                 |
#        | as published by the Free Software Foundation; either version 2              |
#        | of the License, or (at your option) any later version.                      |
#        |                                                                             |
#        | This program is distributed in the hope that it will be useful,             |
#        | but WITHOUT ANY WARRANTY; without even the implied warranty of              |
#        | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
#        | GNU General Public License for more details.                                |
#        |                                                                             |
#        | You should have received a copy of the GNU General Public License           |
#        | along with this program; if not, write to the Free Software                 |
#        | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
#        | http://www.fsf.org/                                                         |
#        +-----------------------------------------------------------------------------+
#


require '/var/efw/header.pl';

my $avuplogs_file = "/var/log/clamav/clam-update.log";
my $conffile = "/var/efw/clamav/settings";
my $conffile_default = "/var/efw/clamav/default/settings";

my %checked = ( 0 => '', 'on' => "checked='checked'" );
my %frequency = ('hourly' => '', 'daily' => '', 'weekly' => '', 'monthly' => '');

my $do_reload = 0;

my %conf_hash = ();
my $conf = \%conf_hash;

my $restart = '/usr/local/bin/restartclamav';

sub setfrequency() {
    $frequency{'hourly'} = '';
    $frequency{'daily'} = '';
    $frequency{'weekly'} = '';
    $frequency{'monthly'} = '';
    $frequency{$conf->{'UPDATE_SCHEDULE'}} = 'checked="checked"';

}

sub loadconfig() {
    if (-e $conffile_default) {
	readhash($conffile_default, $conf);
    }
    if (-e $conffile) {
	readhash($conffile, $conf);
    }
    setfrequency();
}

sub value_or_nbsp($) {
    my $value = shift;
    if ($value =~ /^$/) {
	return '&nbsp;';
    }
    return $value;
}

sub save() {
    my %par;
    getcgihash(\%par);

    if ($par{'action'} eq 'freshclam') {
        `$restart`;
	return (0,'');
    }

    if ($par{'action'} ne 'save') {
	return (0,'');
    }

    if (($par{'MAXSCANSIZE'} !~ /\d+/) or 
	($par{'MAXSCANSIZE'} < 0)){
	return (1,$tr{'invalid maxfilesize'});
    }

    if (($par{'MAXFILESIZE'} !~ /\d+/) or 
	($par{'MAXFILESIZE'} < 0)){
	return (1,$tr{'invalid maxfilesize'});
    }

    if (($par{'MAXRECURSION'} !~ /\d+/) or 
	($par{'MAXRECURSION'} < 0)){
	return (1,$tr{'invalid maxrecursion'});
    }

    if (($par{'MAXFILES'} !~ /\d+/) or 
	($par{'MAXFILES'} < 0)){
	return (1,$tr{'invalid maxfile'});
    }

    if (($par{'MAXFILESIZE'} ne $conf->{'MAXFILESIZE'}) or
	($par{'MAXSCANSIZE'} ne $conf->{'MAXSCANSIZE'}) or
	($par{'UPDATE_SCHEDULE'} ne $conf->{'UPDATE_SCHEDULE'}) or
	($par{'MAXRECURSION'} ne $conf->{'MAXRECURSION'}) or
	($par{'MAXFILES'} ne $conf->{'MAXFILES'}) or
	($par{'ARCHIVE_BLOCK_ENCRYPTED'} ne $conf->{'ARCHIVE_BLOCK_ENCRYPTED'})
	) {


	$conf->{'MAXFILESIZE'}=$par{'MAXFILESIZE'};
	$conf->{'MAXSCANSIZE'}=$par{'MAXSCANSIZE'};
	$conf->{'MAXRECURSION'}=$par{'MAXRECURSION'};
	$conf->{'MAXFILES'}=$par{'MAXFILES'};

	$conf->{'ARCHIVE_BLOCK_ENCRYPTED'}=$par{'ARCHIVE_BLOCK_ENCRYPTED'};

	$conf->{'UPDATE_SCHEDULE'}=$par{'UPDATE_SCHEDULE'};

	writehash($conffile, $conf);
	setfrequency();
	$do_reload = 1;
    }
    if ($do_reload) {
        `$restart`;
    }
    return (0,'');
}

sub display_clamsigantures() {
    my $lastupdate = `tac /var/log/clamav/clamd.log| grep -m1 "Database updated"`;
    my $maincvd = `tac /var/log/clamav/clamd.log| grep -m1 "main.cvd"`;
    my $mainupdated = `tac /var/log/clamav/clamd.log| grep -m1 "main.cvd updated"`;
    my $dailycvd = `tac /var/log/clamav/clamd.log| grep -m1 "daily.cvd"`;
    my $dailyupdated = `tac /var/log/clamav/clamd.log| grep -m1 "daily.cvd updated"`;

    my $updatedate = "";
    my $totalsig = "";
    my $from = "";

    my $maincheckdate = "";
    my $mainsig = "";
    my $mainver = "";
    my $mainupdatedate = "";

    my $dailycheckdate = "";
    my $dailysig = "";
    my $dailyver = "";
    my $dailyupdatedate = "";

#root@vienna:/etc/cron.hourly # tac /var/log/clamav/clamd.log| grep -m1 "Database updated"
#Jun 16 20:01:04 vienna freshclam[11765]: Database updated (60044 signatures) from db.local.clamav.net (IP: 129.27.65.27)
    if ($lastupdate =~ /^(... .. ..:..:..)[^\(]+\(([\d]+) signatures\) from ([^\ ]+)/) {
	$updatedate = $1;
	chomp($updatedate);
	$totalsig = $2;
	chomp($totalsig);
	$from = $3;
	chomp($from);
    }


#root@vienna:/etc/cron.hourly # tac /var/log/clamav/clamd.log| grep -m1 "main.cvd"
#Jun 17 20:01:03 vienna freshclam[30846]: main.cvd is up to date (version: 39, sigs: 58116, f-level: 8, builder: tkojm)
    if ($maincvd =~ /^(... .. ..:..:..).*version: ([\d]+), sigs: ([\d]+)/) {
	$maincheckdate = $1;
	chomp($maincheckdate);
	$mainsig = $2;
	chomp($mainsig);
	$mainver = $3;
	chomp($mainver);
    }


#root@vienna:/etc/cron.hourly # tac /var/log/clamav/clamd.log| grep -m1 "main.cvd updated"
    if ($mainupdated =~ /^(... .. ..:..:..).*/) {
	$mainupdatedate = $1;
	chomp($mainupdatedate);
    }

#root@vienna:/etc/cron.hourly # tac /var/log/clamav/clamd.log| grep -m1 "daily.cvd"
#Jun 17 20:01:03 vienna freshclam[30846]: daily.cvd is up to date (version: 1548, sigs: 1928, f-level: 8, builder: ccordes)
    if ($dailycvd =~ /^(... .. ..:..:..).*version: ([\d]+), sigs: ([\d]+)/) {
	$dailycheckdate = $1;
	chomp($dailycheckdate);
	$dailysig = $2;
	chomp($dailysig);
	$dailyver = $3;
	chomp($dailyver);
    }

#root@vienna:/etc/cron.hourly # tac /var/log/clamav/clamd.log| grep -m1 "daily.cvd updated"
#Jun 16 20:01:04 vienna freshclam[11765]: daily.cvd updated (version: 1548, sigs: 1928, f-level: 8, builder: ccordes)
    if ($dailyupdated =~ /^(... .. ..:..:..).*/) {
	$dailyupdatedate = $1;
	chomp($dailyupdatedate);
    }


    $updatedate = value_or_nbsp($updatedate);
    $totalsig = value_or_nbsp($totalsig);
    $from = value_or_nbsp($from);
    $maincheckdate = value_or_nbsp($maincheckdate);
    $mainsig = value_or_nbsp($mainsig);
    $mainver = value_or_nbsp($mainver);
    $mainupdatedate = value_or_nbsp($mainupdatedate);
    $dailycheckdate = value_or_nbsp($dailycheckdate);
    $dailysig = value_or_nbsp($dailysig);
    $dailyver = value_or_nbsp($dailyver);
    $dailyupdatedate = value_or_nbsp($dailyupdatedate);

    openbox('100%', 'left', $tr{'clamav signatures'});

    $line = $tr{'signature line'};
    $line =~ s/%s/$updatedate/;
    $line =~ s/%s/$from/;
    $line =~ s/%s/$totalsig/;


    print <<END

$line

<br>
<br>
<table cols="5">
  <tr>
    <td width="25%"><b>$tr{'Last signature check'}</b></td>
    <td width="25%"><b>$tr{'Signature type'}</b></td>
    <td width="10%"><b>$tr{'Signature version'}</b></td>
    <td width="10%"><b>$tr{'Signature count'}</b></td>
    <td width="25%"><b>$tr{'Signature last update'}</b></td>
  </tr>

  <tr bgcolor='$table1colour'>
    <td>$maincheckdate</td>
    <td>$tr{'signatures main'}</td>
    <td>$mainsig</td>
    <td>$mainver</td>
    <td>$mainupdatedate</td>
  </tr>

  <tr bgcolor='$table2colour'>
    <td>$dailycheckdate</td>
    <td>$tr{'signatures daily'}</td>
    <td>$dailysig</td>
    <td>$dailyver</td>
    <td>$dailyupdatedate</td>
  </tr>
<tr>
<td>
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
  <input type="hidden" name="action" value="freshclam">
  <input type='submit' name='submit' value="$tr{'freshclam now'}" />
</form>
</td>
<td colspan="3">
  <a href="http://clamav-du.securesites.net/cgi-bin/clamgrok" target="_blank">$tr{'online virus database'}</a>
</td>
<td>&nbsp;</td>
</table>
END
;
    closebox();

}

sub display() {
    openbox('100%', 'left', $tr{'clamav configuration'});
    $statusonly = shift;
    print <<EOF
	<form method='post' action='$ENV{'SCRIPT_NAME'}'>
        <input type="hidden" name="action" value="save">

	<table border='0' cellspacing="0" cellpadding="4" cols="5">
          <tr>
            <td colspan="5"><b>$tr{'Anti archive bombs'}</b></td>
          </tr>

	  <tr>
            <td>$tr{'clamav maxfilesize'}:</td>
            <td>
              <input type="text" name="MAXFILESIZE" value="$conf->{'MAXFILESIZE'}" SIZE="5" MAXLENGTH="5" />
            </td>
            <td>&nbsp;</td>

            <td>$tr{'clamav maxrecursion'}:</td>
            <td>
              <input type="text" name="MAXRECURSION" value="$conf->{'MAXRECURSION'}" SIZE="5" MAXLENGTH="5" />
            </td>
          </tr>

          <tr>
            <td>$tr{'clamav maxscansize'}:</td>
            <td>
              <input type="text" name="MAXSCANSIZE" value="$conf->{'MAXSCANSIZE'}" SIZE="5" MAXLENGTH="5" />
            </td>
            <td>&nbsp;</td>

            <td>$tr{'clamav maxfiles'}:</td>
            <td>
              <input type="text" name="MAXFILES" value="$conf->{'MAXFILES'}" SIZE="5" MAXLENGTH="5" />
            </td>

          </tr>

          <tr valign="top">
            <td>$tr{'block encrypted'}:</td>
            <td>
              <input type="checkbox" name="ARCHIVE_BLOCK_ENCRYPTED" $checked{$conf->{'ARCHIVE_BLOCK_ENCRYPTED'}} value="on" />
            </td>
          </tr>

          <tr>
            <td>&nbsp;</td>
          </tr>
          <tr>
            <td colspan="5"><b>$tr{'Clamav update schedule'}</b></td>
          </tr>

          <tr>
            <td></td>
            <td><input type="radio" name="UPDATE_SCHEDULE" $frequency{'hourly'} value="hourly"> $tr{'hourly'}</td>
            <td><input type="radio" name="UPDATE_SCHEDULE" $frequency{'daily'} value="daily"> $tr{'daily'}</td>
            <td><input type="radio" name="UPDATE_SCHEDULE" $frequency{'weekly'} value="weekly"> $tr{'weekly'}</td>
            <td><input type="radio" name="UPDATE_SCHEDULE" $frequency{'monthly'} value="monthly"> $tr{'monthly'}</td>
          </tr>
          <tr colspan="5">
            <td align="center">
              <input type='submit' name='submit' value='$tr{save}' />
            </td>
            <td>&nbsp;</td>
          </tr>

        </table>
        </form>
EOF
;
    closebox();
}


sub display_error($$) {
    my $error = shift;
    my $errormessage = shift;
    if ($errormessage !~ /^$/) {
	&openbox('100%', 'left', $tr{'error messages'});
	print "<CLASS name='base'>$errormessage\n";
	print "&nbsp;</CLASS>\n";
	&closebox();
    }
}

