#!/usr/bin/perl

#
#        +-----------------------------------------------------------------------------+
#        | Endian Firewall                                                             |
#        +-----------------------------------------------------------------------------+
#        | Copyright (c) 2005-2006 Endian                                              |
#        |         Endian GmbH/Srl                                                     |
#        |         Bergweg 41 Via Monte                                                |
#        |         39057 Eppan/Appiano                                                 |
#        |         ITALIEN/ITALIA                                                      |
#        |         info@endian.it                                                      |
#        |                                                                             |
#        | This program is free software; you can redistribute it and/or               |
#        | modify it under the terms of the GNU General Public License                 |
#        | as published by the Free Software Foundation; either version 2              |
#        | of the License, or (at your option) any later version.                      |
#        |                                                                             |
#        | This program is distributed in the hope that it will be useful,             |
#        | but WITHOUT ANY WARRANTY; without even the implied warranty of              |
#        | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               |
#        | GNU General Public License for more details.                                |
#        |                                                                             |
#        | You should have received a copy of the GNU General Public License           |
#        | along with this program; if not, write to the Free Software                 |
#        | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. |
#        | http://www.fsf.org/                                                         |
#        +-----------------------------------------------------------------------------+
#


require '/var/efw/header.pl';
use POSIX();

my %cgiparams;
my %logsettings;

&getcgihash(\%cgiparams);
$logsettings{'LOGVIEW_REVERSE'} = 'off';
&readhash("${swroot}/logging/settings", \%logsettings);

if ($logsettings{'LOGVIEW_SIZE'} =~ /^\d+$/) {
    $viewsize = $logsettings{'LOGVIEW_SIZE'};
}

my $offset = 1;
my $fileoffset = 0;

if ($cgiparams{'OFFSET'} =~ /\d+/) {
    $offset = $cgiparams{'OFFSET'};
}
if ($cgiparams{'FILEOFFSET'} =~ /\d+/) {
    $fileoffset = $cgiparams{'FILEOFFSET'};
}
my $filter = $cgiparams{'FILTER'};

my @files = glob("/var/log/maillog*");
my $filetotal=scalar(@files);

if ($fileoffset > $filetotal) {
    $fileoffset = 0;
}

my $filestr="/var/log/clamav/clamd.log";
if ($fileoffset > 0) {
    $filestr="/var/log/clamav/clamd.${fileoffset}.gz";
}


if (!(open (FILE,($filestr =~ /.gz$/ ? "gzip -dc $filestr |" : $filestr)))) {
    $errormessage = "$tr{'date not in logs'}: $filestr $tr{'could not be opened'}";
}

my $lines = 0;
my @log = ();

if (!$skip) {
    foreach my $line (<FILE>) {
	if ($line !~ /(^... .. ..:..:..) [\w\-]+ (.*)$/) {
	    next;
	}
	if (($filter !~ /^$/) && ($line !~ /$filter/)) {
	    next;
	}
	$log[$lines] = $line;
	$lines++;
    }
    close (FILE);	
}

if ($cgiparams{'ACTION'} eq $tr{'export'}) {
	print "Content-type: text/plain\n\n";
	print "Endian Firewall CLAMD log\r\n\r\n";
	if ($logsettings{'LOGVIEW_REVERSE'} eq 'on') { @log = reverse @log; }

	foreach my $line (@log) {
	    $line =~ /(^... .. ..:..:..) [\w\-]+ (.*)$/;
	    my $timestamp = $1; my $packet = $2;
	    print "$timestamp $packet\r\n";
	}
	exit 0;
}

&showhttpheaders();

&openpage($tr{'SMTP log'}, 1, '');

&openbigbox('100%', 'left');

if ($errormessage) {
    &openbox('100%', 'left', $tr{'error messages'});
    print "<font class='base'>$errormessage&nbsp;</font>\n";
    &closebox();
}

&openbox('100%', 'left', "$tr{'settings'}:");

print <<END
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<table width='100%'>
  <tr>
    <td width='5%' class='base'>$tr{'filter'}:</td>
    <td width='30%'><input type="text" size="50" name='FILTER' VALUE="$filter"></td>

    <td width='15%' align='right' class='base'>$tr{'jump to file'}:</td>
    <td width='5%'><input type="text" SIZE="4" name='FILEOFFSET' VALUE="$fileoffset"></td>

    <td width='15%' align='right' class='base'>$tr{'jump to offset'}:</td>
    <td width='5%'><input type="text" SIZE="4" name='OFFSET' VALUE="$offset"></td>

    <td width='10%' align='center'><input type='submit' name='ACTION' value='$tr{'update'}' /></td>
    <td width='10%' align='center'><input type='submit' name='ACTION' value='$tr{'export'}' /></td>
  </tr>
</table>
</form>
END
;

&closebox();

my $totaloffset=POSIX::ceil($lines/$viewsize);

&openbox('100%', 'left', $tr{'log'});
print "<p><b>$tr{'log hits'}:  $lines - $tr{'fileoffset'}: $fileoffset/$filetotal - $tr{'offset'}: $offset/$totaloffset</b></p>";

my $start = $lines - ($viewsize * $offset);
my $prev = $offset+1;
my $next = $offset-1;
my $prevfile = $fileoffset;
my $nextfile = $fileoffset;

if ($start <= 0) { 
    $start = 0;
    $prev = 1;
    $prevfile++;
}
if ($next < 1) {
    $next = 1;
    $nextfile--;
    if ($nextfile < 0) {
	$next = -1;
    }
}

&oldernewer();
print <<END
<table width='100%'>
<tr>
    <td width='11%' align='center' class='boldbase'><b>$tr{'time'}</b></td>
    <td align='center' class='boldbase'><b>$tr{'data'}</b></td>
</tr>
END
;

my @slice = splice(@log, $start, $viewsize);

if ($logsettings{'LOGVIEW_REVERSE'} eq 'on') { @slice = reverse @slice; }

my $i = 0;
foreach my $line (@slice) {
    $line =~ /(^... .. ..:..:..) [\w\-]+ (.*)$/;
    my $timestamp = $1; my $packet = $2;
    if ($i % 2) {
	print "<tr bgcolor='$table1colour'>\n";
    } else {
	print "<tr bgcolor='$table2colour'>\n";
    }

    print <<END
	<td>$timestamp</td>
	<td>$packet</td>
        </tr>
END
;
    $i++;
}

print "</table>";

&oldernewer();

&closebox();

&closebigbox();

&closepage();

sub oldernewer {
    print <<END
<table width='100%'>
  <tr>
END
;

    print "<td align='center' width='50%'>";
    if ($prev != -1) {
print <<END
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
    <input type="hidden" name="OFFSET" value="$prev">
    <input type="hidden" name="FILEOFFSET" value="$prevfile">
    <input type="hidden" name="FILTER" value="$filter">
    <input type="submit" name="ACTION" value="$tr{'older'}">
</form>
END
;
    } else {
	print "$tr{'older'}";
    }
    print "</td>\n";

    print "<td align='center' width='50%'>";
    if ($next != -1) {
print <<END
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
    <input type="hidden" name="OFFSET" value="$next">
    <input type="hidden" name="FILEOFFSET" value="$nextfile">
    <input type="hidden" name="FILTER" value="$filter">
    <input type="submit" name="ACTION" value="$tr{'newer'}">
</form>
END
;
    } else {
	print "$tr{'newer'}";
    }
    print "</td>\n";

    print <<END
</tr>
</table>
END
;
}
